@@ -1,5 +1,5 @@
 /* GENIUS Calculator
- * Copyright (C) 1997-2008 Jiri (George) Lebl
+ * Copyright (C) 1997-2009 Jiri (George) Lebl
  *
  * Author: Jiri (George) Lebl
  *
@@ -68,8 +68,10 @@
 
 /*Globals:*/
 
+const gboolean genius_is_gui = FALSE;
+
 /*calculator state*/
-calcstate_t curstate={
+GelCalcState curstate={
 	128,
 	12,
 	FALSE,
@@ -83,11 +85,12 @@ calcstate_t curstate={
 	20, /* chop */
 	5 /* chop_when */
 	};
+
+const GelHookFunc gel_evalnode_hook = NULL;
+const GelHookFunc _gel_tree_limit_hook = NULL;
 	
 extern int parenth_depth;
 
-extern int interrupted;
-
 static int use_readline = TRUE;
 
 static gboolean genius_in_dev_dir = FALSE;
@@ -147,9 +150,13 @@ gel_call_help (const char *function)
 	g_free (str);
 
 	if G_UNLIKELY (access (file, R_OK) != 0) {
-		puterror (_("Cannot locate the manual"));
 		g_free (file);
-		return;
+		file = g_build_filename (DATADIR, "genius", "genius.txt", NULL);
+		if G_UNLIKELY (access (file, R_OK) != 0) {
+			puterror (_("Cannot locate the manual"));
+			g_free (file);
+			return;
+		}
 	}
 
 	str = g_find_program_in_path ("less");
@@ -161,6 +168,7 @@ gel_call_help (const char *function)
 
 		argv[0] = str;
 		argv[1] = file;
+		argv[2] = NULL;
 		g_spawn_sync  (NULL /* wd */,
 			       argv,
 			       NULL /* envp */,
@@ -190,19 +198,27 @@ gel_call_help (const char *function)
 }
 
 char *
-gel_ask_string (const char *query)
+gel_ask_string (const char *query, const char *def)
 {
 	char *txt = NULL;
 
 	g_print ("\n%s\n", ve_sure_string (query));
 	if (use_readline) {
-		char *s = readline (">");
+		char *s;
+		if ( ! ve_string_empty (def)) {
+			const char *p;
+			for (p = def; *p != '\0'; p++)
+				rl_stuff_char (*p);
+		}
+		s = readline (">");
 		if (s != NULL) {
 			txt = g_strdup (s);
 			free (s);
 		}
 	} else {
 		char buf[256];
+		if ( ! ve_string_empty (def))
+			g_print (_("Suggested: %s\n"), ve_sure_string (def));
 		if (fgets (buf, sizeof (buf), stdin) != NULL) {
 			int len = strlen (buf);
 			if (buf[len-1] == '\n')
@@ -239,8 +255,8 @@ get_term_width (GelOutput *gelo)
 }
 
 
-static void
-set_state(calcstate_t state)
+void
+gel_set_state(GelCalcState state)
 {
 	curstate = state;
 
@@ -248,15 +264,15 @@ set_state(calcstate_t state)
 	    state.output_style == GEL_OUTPUT_LATEX ||
 	    state.output_style == GEL_OUTPUT_MATHML ||
 	    state.output_style == GEL_OUTPUT_TROFF)
-		gel_output_set_length_limit (main_out, FALSE);
+		gel_output_set_length_limit (gel_main_out, FALSE);
 	else
-		gel_output_set_length_limit (main_out, TRUE);
+		gel_output_set_length_limit (gel_main_out, TRUE);
 }
 
 static void
 interrupt (int sig)
 {
-	interrupted = TRUE;
+	gel_interrupted = TRUE;
 	if (use_readline)
 		rl_stuff_char ('\n');
 	signal (SIGINT, interrupt);
@@ -289,7 +305,8 @@ main(int argc, char *argv[])
 	gboolean be_quiet = FALSE;
 	char *exec = NULL;
 
-	genius_is_gui = FALSE;
+	g_set_prgname ("genius");
+	g_set_application_name (_("Genius"));
 
 	/* kind of a hack to find out if we are being run from the
 	 * directory we were built in */
@@ -314,8 +331,6 @@ main(int argc, char *argv[])
 
 	signal (SIGINT, interrupt);
 
-	statechange_hook = set_state;
-
 	for(i=1;i<argc;i++) {
 		int val;
 		if(lastarg || argv[i][0]!='-')
@@ -428,7 +443,7 @@ main(int argc, char *argv[])
 			g_print (_("Genius %s\n"
 				   "%s%s\n"),
 				 VERSION,
-				 COPYRIGHT_STRING,
+				 GENIUS_COPYRIGHT_STRING,
 				 get_version_details ());
 			exit (0);
 		} else {
@@ -470,21 +485,6 @@ main(int argc, char *argv[])
 		exit (1);
 	}
 
-#if 0
-	{
-		/* FIXME: use this for option parsing,
-		 * we really only need gnome-program for
-		 * gnome_config, perhaps we should switch to
-		 * ve-config anyway */
-		char *fakeargv[] = { argv[0], NULL };
-		gnome_program_init ("genius", VERSION, 
-				    LIBGNOME_MODULE /* module_info */,
-				    1, fakeargv,
-				    /* GNOME_PARAM_POPT_TABLE, options, */
-				    NULL);
-	}
-#endif
-
 	gel_read_plugin_list();
 
 	if (do_compile || do_gettext)
@@ -498,22 +498,22 @@ main(int argc, char *argv[])
 			   "For license details type `warranty'.\n"
 			   "For help type 'manual' or 'help'.%s\n\n"),
 			 VERSION,
-			 COPYRIGHT_STRING,
+			 GENIUS_COPYRIGHT_STRING,
 			 get_version_details ());
 		be_quiet = FALSE;
 	}
 
-	main_out = gel_output_new();
+	gel_main_out = gel_output_new();
 	if(!be_quiet)
-		gel_output_setup_file(main_out, stdout, 80,
+		gel_output_setup_file(gel_main_out, stdout, 80,
 				      get_term_width);
 	else
-		gel_output_setup_black_hole(main_out);
+		gel_output_setup_black_hole(gel_main_out);
 
 
-	set_new_calcstate(curstate);
-	set_new_errorout(calc_puterror);
-	set_new_infoout(puterror);
+	gel_set_new_calcstate(curstate);
+	gel_set_new_errorout(calc_puterror);
+	gel_set_new_infoout(puterror);
 
 	/*init the context stack and clear out any stale dictionaries
 	  except the global one, if this is the first time called it
@@ -537,6 +537,14 @@ main(int argc, char *argv[])
 						       "gel",
 						       "lib.cgel",
 						       NULL);
+			if (access (file, F_OK) != 0) {
+				g_free (file);
+				file = g_build_filename (DATADIR,
+							 "genius",
+							 "gel",
+							 "lib.cgel",
+							 NULL);
+			}
 			gel_load_compiled_file (NULL,
 						file,
 						FALSE);
@@ -605,7 +613,7 @@ main(int argc, char *argv[])
 
 	if (exec != NULL) {
 		line_len_cache = -1;
-		gel_evalexp (exec, NULL, main_out, NULL, FALSE, NULL);
+		gel_evalexp (exec, NULL, gel_main_out, NULL, FALSE, NULL);
 		gel_test_max_nodes_again ();
 		line_len_cache = -1;
 		goto after_exec;
@@ -619,14 +627,14 @@ main(int argc, char *argv[])
 				line_len_cache = -1;
 				e = get_p_expression();
 				line_len_cache = -1;
-				if(e) gel_evalexp_parsed(e,main_out,"= ",TRUE);
+				if(e) gel_evalexp_parsed(e,gel_main_out,"= ",TRUE);
 				gel_test_max_nodes_again ();
 				line_len_cache = -1;
 			} else {
 				line_len_cache = -1;
-				gel_evalexp(NULL, fp, main_out, NULL, FALSE, NULL);
+				gel_evalexp(NULL, fp, gel_main_out, NULL, FALSE, NULL);
 				line_len_cache = -1;
-				if (interrupted)
+				if G_UNLIKELY (gel_interrupted)
 					gel_got_eof = TRUE;
 			}
 			if(inter)
