@@ -1,5 +1,5 @@
-/* GnomENIUS Calculator
- * Copyright (C) 1999 the Free Software Foundation.
+/* GENIUS Calculator
+ * Copyright (C) 1997-2002 George Lebl
  *
  * Author: George Lebl
  *
@@ -22,19 +22,21 @@
 #ifndef _PLUG_API_H_
 #define _PLUG_API_H_
 
-typedef struct _PluginInfo PluginInfo;
-struct _PluginInfo {
+typedef struct _GelPluginInfo GelPluginInfo;
+struct _GelPluginInfo {
 	void (*open)(void); /*open the plugin (this happens every
 			      time the user selects the menuitem)*/
-	int (*save_state)(char *prefix); /*save state using gnome_config
-					   in this file, and return TRUE
-					   if genius should reload this
-					   plugin next time*/
-	void (*restore_state)(char *prefix);
+
+	gboolean (*save_state) (const char *unique_id);
+			    /* return TRUE if genius should reload this
+			       plugin next time.  Note that the unique id
+			       can be used if ever multiple sessions are
+			       implemented */
+	void (*restore_state) (const char *unique_id);
 };
 
 /*this is here to avoid "prototype warnings", this is a function which the
   plugins should define*/
-PluginInfo * init_func(void);
+GelPluginInfo * init_func(void);
 
 #endif
